import fs from "fs-extra";
import { join } from "path";
import walkSync from "walk-sync";
import { cloneDeep, set } from "lodash-es";
import {
  JSON_PATTERN,
  DIR_ERROR_MESSAGE,
  DEFAULT_OUT_DIR,
  JSON_ERROR_MESSAGE,
} from "./constants.js";
import { diff } from "./diff.js";

export const isDirectory = (path) => {
  const stat = fs.lstatSync(path);
  return stat.isDirectory();
};

export const isJson = (path) => {
  return JSON_PATTERN.test(path);
};

export const getDirErrorMessage = (prefix) => {
  return `${prefix} ${DIR_ERROR_MESSAGE}`;
};

export const getJsonErrorMessage = (prefix) => {
  return `${prefix} ${JSON_ERROR_MESSAGE}`;
};

const getDiffJson = (oldStruct, newStruct) => {
  const result = diff(oldStruct, newStruct);
  // 合并新增和修改的项
  let target = {};

  result.added.forEach((item) => {
    const [keyPath, value] = item;
    set(target, keyPath, value);
  });

  result.edited.forEach((item) => {
    const [keyPath, _oldValue, newValue] = item;
    set(target, keyPath, newValue);
  });

  return target;
};

/**
 * @description 比对两个相同结构的目录，如果传入 output
 */
export const diffDir = (oldDirPath, newDirPath, option) => {
  const basePath = process.cwd();
  const jsonFileStructs = walkSync(oldDirPath)
    .filter((filePath) => JSON_PATTERN.test(filePath))
    .map((filePath) => {
      const pathStruct = {
        basePath: "",
        filePath: "",
        oldDirPath: "",
        newDirPath: "",
      };
      pathStruct["basePath"] = basePath;
      pathStruct["filePath"] = filePath;
      pathStruct["oldDirPath"] = oldDirPath;
      pathStruct["newDirPath"] = newDirPath;

      return pathStruct;
    });

  for (const {
    basePath,
    filePath,
    oldDirPath,
    newDirPath,
  } of jsonFileStructs) {
    const oldJsonStruct = fs.readJSONSync(
      join(basePath, oldDirPath, filePath),
      {
        throws: false,
      }
    );
    const newJsonStruct = fs.readJSONSync(
      join(basePath, newDirPath, filePath),
      {
        throws: false,
      }
    );

    const result = getDiffJson(oldJsonStruct, newJsonStruct);
    const outputPath = option.output
      ? join(basePath, option.output, filePath)
      : join(basePath, DEFAULT_OUT_DIR, filePath);

    // output
    fs.outputJSONSync(outputPath, result, { spaces: 2 });
  }
};

export const diffJson = (oldStructPath, newStructPath) => {
  const basePath = process.cwd();
  const oldJsonStruct = fs.readJSONSync(join(basePath, oldStructPath), {
    throws: false,
  });
  const newJsonStruct = fs.readJSONSync(join(basePath, newStructPath), {
    throws: false,
  });
};
